
# coding: utf-8

# In[1]:

#Various setup commands

get_ipython().magic('matplotlib inline')

from numpy import *
from scipy import *
import time
from scipy import stats
from matplotlib import *
from matplotlib.pyplot import *

import seaborn as sns
sns.set(style='ticks', palette='Set2')
sns.despine()

rc('axes', linewidth=3)
rc('xtick', labelsize=22)
rc('xtick.major', width=3)
rc('xtick.major', size=10)
rc('ytick', labelsize=22)
rc('ytick.major', size=10)
rc('ytick.major', width=3)
rc('lines', linewidth=4)


# In[2]:

#Define Cell class, parallelAdder simulation, and chModel simulation
#See Methods in addition to inline comments for details.
#
#
#

import copy

#-----Define class Cell to be used in simulation of parallel adder below
class Cell:
    #-----Properties of the cell
    t = 0. #time
    v = 0. #size
    a = 0. #alternator/accelerator
    rate = 0. #growth rate
    oris = 2 #number of origins
    divr = 0. #division ratio
    
    #-----For keeping track of events
    vNextInit = 0. #size at next initiation
    vNextDivs = [0.]  #sizes at next divisions, only for parallel adder
    tNextDivs = [0.]  #time to next divisions, only for CH
    
    vOfInits = [0.] #size at previous initiations
    tOfInits = [0.] #time at previous initiations
    
    #-----For bookkeeping purposes
    vb = 0. #size at birth
    vi = 0. #size at initiation
    vd = 0. #size at division
    tLastDiv = 0. #time of last division
    tLastInit = 0. #time of last initiation
    
    def __init__(self, v, a):
        self.t = 0.
        self.td = 0.
        self.v = v
        self.vb = v
        self.vi = v
        self.vd = v
        self.rate = 0.
        self.vNextInit = 0.
        self.vNextDivs = [0.]
        self.a = a
        self.oris = 2
        self.tLastDiv = 0.
        self.tLastInit = 0.
        self.vOfInits = [v]
        self.tOfInits = [0.]
        self.divr = 0.

#-----Parallel adder simulation
def parallelAdder(r, cvr, cvt, cvl, cve, cvt2, bid0, bid1, bii0, bii1):
    #bid0: increment from initiation to division for alternators
    #bii1: increment from initiation to initiation for accelerators, etc...
    
    #Initialize-----------------------------------------------------------------
    cells = []
    
    #-----Outputs
    vbs = []
    vds = []
    vis = []
    aas = [] #alternator or accelerator
    tds = []
    cds = []
    divrs = []
    
    #----- "+r" Outputs for calculating statistics involving initiations, see below.
    vbsr = []
    visr = []
    vdsr = []
    aasr = []
    tdsr = []
    cdsr = []
    divrsr = []
    
    lmin = log(2)/100
    #-----Initial alternator
    x = Cell(1,0)
    x.rate = max(log(2) + random.normal(0,cvl),lmin) #prevents negative growth rate in the very rare cases
    x.vNextInit = (x.v + x.oris*bii1)*exp(x.rate*random.normal(0,cvt)) #adder between initiations, with time additive noise
    x.vNextDivs[0] = (x.v + x.oris*bid1)*exp(x.rate*random.normal(0,cvt2)) #adder from init to div, with time additive noise
    x.divr = r + random.normal(0,cvr)
    cells.append(x)
    
    #-----Initial accelerator
    x = Cell(1,1)
    x.rate = max(log(2) + random.normal(0,cvl),lmin)
    x.vNextInit = (x.v + x.oris*bii0)*exp(x.rate*random.normal(0,cvt))
    x.vNextDivs[0] = (x.v + x.oris*bid0)*exp(x.rate*random.normal(0,cvt2))
    x.divr = r + random.normal(0,cvr)
    cells.append(x)
    
    #Simulate-----------------------------------------------------------------
    #A population growing in time beginning with one alternator and one accelerator
    tic = time.time()
    #-----Advance in time for #gens generations
    gens = 13
    tStep = 0.01 #in units of generations
    simTime = 0.
    while simTime < gens:
        #-----Step
        simTime = simTime + tStep
        for x in cells:
            x.t = x.t + tStep
            x.v = x.v*exp(tStep*x.rate) #grow exponentially
            
        cellsNew = []
        #-----Perform events
        for x in cells:
            #-----Initiate
            #Upon accumulating enough volume
            if x.v - x.vNextInit >= 0:
                #-----Record event
                x.tLastInit = x.t
                x.vi = x.v
                x.vOfInits.append(x.v)
                x.tOfInits.append(x.t)
                
                #-----Update cell
                x.oris = 2*x.oris #double number of origins

                #-----Calculate vNextInit and vNextDiv according to parallel adder
                if x.a == 1: #if accelerator
                    x.vNextInit = (x.v + x.oris*bii1)*exp(x.rate*random.normal(0,cvt))
                    x.vNextDivs.append((x.v + x.oris*bid1)*exp(x.rate*random.normal(0,cvt2)))
                else: #if alternator
                    x.vNextInit = (x.v + x.oris*bii0)*exp(x.rate*random.normal(0,cvt))
                    x.vNextDivs.append((x.v + x.oris*bid0)*exp(x.rate*random.normal(0,cvt2)))

            #-----Divide
            #Upon accumulating enough volume
            if x.vNextDivs and x.v - x.vNextDivs[0] >= 0:
                #-----Record event and output
                tds.append(x.t - x.tLastDiv)
                vbs.append(x.vb + random.normal(0,cve)) #with size additive measurement error
                vds.append(x.v + random.normal(0,2*cve))
                vis.append(x.vOfInits[0] + random.normal(0,cve))
                aas.append(x.a)
                cds.append(x.t - x.tOfInits[0])
                if x.a == 1:
                    divrs.append(x.divr)
                else:
                    divrs.append(1-x.divr)
                
                #-----For statistics involving initiations, do not record extra initiations,
                #or cell cycles with E period. This is the case when the time of last initiation
                #is greater than the time of last division, and it is the only initiation
                #in the current cycle.
                if len(x.tOfInits) == 1 and x.tLastDiv < x.tLastInit:
                    cdsr.append(x.t - x.tLastInit)
                    visr.append(x.vi + random.normal(0,cve))
                    vdsr.append(x.v  + random.normal(0,2*cve))
                    vbsr.append(x.vb + random.normal(0,cve))
                    tdsr.append(x.t - x.tLastDiv)
                    aasr.append(x.a)
                    if x.a == 1:
                        divrsr.append(x.divr)
                    else:
                        divrsr.append(1 - x.divr)
                
                x.vOfInits.pop(0)
                x.tOfInits.pop(0)

                #-----Update cell
                #----------Accelerator daughter
                x.tLastDiv = x.t
                x.vd = x.v
                x.v = x.vd*x.divr
                x.vb = x.v
                x.rate = max(log(2) + random.normal(0,cvl),lmin)
                x.oris = x.oris/2
                x.vNextDivs.pop(0)
                x.a = 1
                #----------Update vNextInit and vNextDiv (see Methods)
                x.vNextInit = x.vNextInit/2 + (r-1/2)*x.vi
                if x.vNextDivs:
                    x.vNextDivs = [x.vNextDivs[i]/2 + (x.divr-1/2)*x.vOfInits[i] for i in range(len(x.vNextDivs))]
                
                #----------Alternator daughter
                y = copy.deepcopy(x)
                y.v = y.v/y.divr*(1-y.divr)
                y.vb = y.v
                y.rate = max(log(2) + random.normal(0,cvl),0)
                y.a = 0
                #----------Update vNextInit and vNextDiv (see methods)
                y.vNextInit = y.vNextInit + (1-2*y.divr)*y.vi
                if y.vNextDivs:
                    y.vNextDivs = [y.vNextDivs[i] - 2*(y.divr-1/2)*y.vOfInits[i] for i in range(len(y.vNextDivs))]
                y.divr = r + random.normal(0,cvr)
                cellsNew.append(y)
                x.divr = r + random.normal(0,cvr)

        if cellsNew:
            cells = cells + cellsNew
            
    toc = time.time()
    #print("Elapsed time =", toc-tic)
    
    return vbs, vis, vds, aas, tds, cds, divrs, vbsr, visr, vdsr, aasr, tdsr, cdsr, divrsr

#-----CH model simulation
#-----Mostly very similar to parallel adder, except for how division is determined
def chModel(r, cvr, cvt, cvl, cve, cvt2, cd, cvcd, bii0, bii1):
    sigcd = cd*cvcd
    
    #Initialize-----------------------------------------------------------------
    cells = []

    vbs = []
    vds = []
    vis = []
    aas = []
    tds = []
    cds = []
    divrs = []
    
    vbsr = []
    visr = []
    vdsr = []
    aasr = []
    tdsr = []
    cdsr = []
    divrsr = []
    
    lmin = log(2)/100
    
    #-----Initial alternator
    x = Cell(1,0)
    x.rate = max(log(2) + random.normal(0,cvl),lmin)
    x.vNextInit = (x.v + x.oris*bii1)*exp(x.rate*random.normal(0,cvt))
    x.tNextDivs[0] = cd + random.normal(0,sigcd)
    x.divr = r + random.normal(0,cvr)
    cells.append(x)
    
    #-----Initial accelerator
    x = Cell(1,1)
    x.rate = max(log(2) + random.normal(0,cvl),lmin)
    x.vNextInit = (x.v + x.oris*bii0)*exp(x.rate*random.normal(0,cvt))
    x.tNextDivs[0] = cd + random.normal(0,sigcd)
    x.divr = r + random.normal(0,cvr)
    cells.append(x)
    
    #Simulate-----------------------------------------------------------------
    tic = time.time()
    #-----Advance in time
    gens = 13
    tStep = 0.01
    simTime = 0.
    while simTime < gens:
        #-----Step
        simTime = simTime + tStep
        for x in cells:
            x.t = x.t + tStep
            x.v = x.v*exp(tStep*x.rate)
            #-----Keep track of time until division
            x.tNextDivs = [x.tNextDivs[i] - tStep for i in range(len(x.tNextDivs))]
            
        cellsNew = []
        #-----Perform events
        for x in cells:
            #-----Initiate
            if x.v - x.vNextInit >= 0:
                #Record event
                x.tLastInit = x.t
                x.vi = x.v
                x.vOfInits.append(x.v)
                x.tOfInits.append(x.t)
                
                #Update cell
                x.oris = 2*x.oris
                
                #Calculate vNextInit and vNextDiv
                if x.a == 1:
                    x.vNextInit = (x.v + x.oris*bii1)*exp(x.rate*random.normal(0,cvt))
                    x.tNextDivs.append(cd + random.normal(0,sigcd))
                else:
                    x.vNextInit = (x.v + x.oris*bii0)*exp(x.rate*random.normal(0,cvt))
                    x.tNextDivs.append(cd + random.normal(0,sigcd))

            #-----Divide
            if x.tNextDivs and x.tNextDivs[0] <= 0:
                #Record event
                tds.append(x.t - x.tLastDiv)
                vbs.append(x.vb + random.normal(0,cve))
                vds.append(x.v + random.normal(0,2*cve))
                vis.append(x.vOfInits[0] + random.normal(0,cve))
                aas.append(x.a)
                cds.append(x.t - x.tOfInits[0])
                if x.a == 1:
                    divrs.append(x.divr)
                else:
                    divrs.append(1-x.divr)
                
                #Record for initiation, see above
                if len(x.tOfInits) == 1 and x.tLastDiv < x.tLastInit:
                    cdsr.append(x.t - x.tLastInit)
                    visr.append(x.vi + random.normal(0,cve))
                    vdsr.append(x.v  + random.normal(0,2*cve))
                    vbsr.append(x.vb + random.normal(0,cve))
                    tdsr.append(x.t - x.tLastDiv)
                    aasr.append(x.a)
                    if x.a == 1:
                        divrsr.append(x.divr)
                    else:
                        divrsr.append(1 - x.divr)
                
                x.vOfInits.pop(0)
                x.tOfInits.pop(0)

                #Update cells
                #-----Accelerator daughter
                x.tLastDiv = x.t
                x.vd = x.v
                x.v = x.vd*x.divr
                x.vb2 = x.vb
                x.vb = x.v
                x.rate = max(log(2) + random.normal(0,cvl),lmin)
                x.oris = x.oris/2
                x.tNextDivs.pop(0)
                x.a = 1
                #Update vNextInit
                x.vNextInit = x.vNextInit/2 + (r-1/2)*x.vi
                
                #-----Alternator daughter
                y = copy.deepcopy(x)
                y.v = y.v/y.divr*(1-y.divr)
                y.vb = y.v
                y.rate = max(log(2) + random.normal(0,cvl),lmin)
                y.a = 0
                y.vNextInit = y.vNextInit + (1-2*y.divr)*y.vi
                y.divr = r + random.normal(0,cvr)
                cellsNew.append(y)
                x.divr = r + random.normal(0,cvr)
                
        if cellsNew:
            cells = cells + cellsNew
            
    toc = time.time()
    #print("Elapsed time =", toc-tic)
    
    return vbs, vis, vds, aas, tds, cds, divrs, vbsr, visr, vdsr, aasr, tdsr, cdsr, divrsr


# In[3]:

#Define functions to analyze outputs
#
#
#
#

def cv(x):
    return std(x)/mean(x)

#-----Display CVs and correlations
def displayNames():
    print("M's vb vi vd td cd ls")
    print("CV's vb vi vd td cd ls rs")
    print("CC's vbvd vbtd vbvib vivdi vicd tdcd vbls")
    return

def displayStats(vbsr, visr, vdsr, aasr, tdsr, cdsr, divrsr):
    rates = [log(vdsr[i]/vbsr[i])/tdsr[i] for i in range(len(vdsr))]
    
    n = len(vdsr)
    vnorm = mean(vbsr)
    tnorm = mean(tdsr)
    
    #-----Calculate CVs
    cvvb = cv(vbsr)
    cvvi = cv(visr)
    cvvd = cv(vdsr)
    cvtd = cv(tdsr)
    cvcd = cv(cdsr)
    cvls = cv(rates)
    cvrs = cv(divrsr)
    cvbvd = stats.linregress(vbsr,vdsr)[2]
    cvbtd = stats.linregress(vbsr,tdsr)[2]
    cvivdi = stats.linregress(visr,[vdsr[i]-visr[i] for i in range(len(vdsr))])[2]
    cvbvib = stats.linregress(vbsr,[visr[i]-vbsr[i] for i in range(len(vdsr))])[2]
    cvicd = stats.linregress(visr,cdsr)[2]
    ctdcd = stats.linregress(tdsr,cdsr)[2]
    cvbls = stats.linregress(vbsr,rates)[2]
    
    print("M's {:.2f} {:.2f} {:.2f} {:.2f} {:.2f} {:.2f}".format
            (mean(vbsr)/vnorm, mean(visr)/vnorm, mean(vdsr)/vnorm,
             mean(tdsr)/tnorm, mean(cdsr)/tnorm, mean(rates)/log(2)*tnorm))
    print("CV's {:.2f} {:.2f} {:.2f} {:.2f} {:.2f} {:.2f}".format
            (cvvb,cvvi,cvvd,cvtd,cvcd,cvls))
    print("CC's {:.2f} {:.2f} {:.2f} {:.2f} {:.2f} {:.2f} {:.2f}".format
            (cvbvd,cvbtd,cvbvib,cvivdi,cvicd,ctdcd,cvbls))
    
    return


# In[4]:

#-----Set parameters
r = 0.56
cvr = 0.03
cvt = 0.10
cvl = 0.10
cve = 2*0.09
bid0 = 0.87
bid1 = 1.11
bii0 = 0.89
bii1 = 1.13

#Only for BCG
cvt2 = 1e-12

#For CH model
cd = 0.978
cvcd = 0.2

#-----Run simulation
#Parallel adder:
#For 37C, the best fit parameters [cvt, cvl, cve] are [0.10, 0.10, 2*0.09]
#The set parameters are [r, cvr, bid0, bid1, bii0, bii1] = [0.56, 0.03, 0.87, 1.11, 0.89, 1.13]
#The size increments are proportionally distributed according to r
vbs, vis, vds, aas, tds, cds, divrs, vbsr, visr, vdsr, aasr, tdsr, cdsr, divrsr = parallelAdder(r, cvr, cvt, cvl, cve, cvt2, bid0, bid1, bii0, bii1)

#Birth-division adder: set [bid0, bid1, bii0, bii1] = [0, 0, delta0, delta1] (see Methods) and use parallelAdder
#For 37C, the best fit parameters [cvt, cvl, cve] are [0.06, 0.13, 2*0.09]
#and the delta's are proportioned according to r, so delta0 = 1.76, and delta1 = 2.24

#CH model:
#Adjust the noises, but leave the increments the same
#For 37C, the best fit parameters [cvt, cvl, cve] are [0.08, 0.05, 2*0.03]
#vbs, vis, vds, aas, tds, cds, divrs, vbsr, visr, vdsr, aasr, tdsr, cdsr, divrsr = chModel(r, cvr, cvt, cvl, cve, cvt2, cd, cvcd, bii0, bii1)

#-----Display results
print([r, cvr, cvt, cvl, cve/2, bid0, bid1, bii0, bii1, cvt2])
displayNames()

#Use below for stats involving initiations (see above and Methods)
print("")
displayStats(vbsr, visr, vdsr, aasr, tdsr, cdsr, divrsr)

#Use below for all other stats
print("")
displayStats(vbs, vis, vds, aas, tds, cds, divrs)


# In[5]:

#-----Read in data
import csv
with open('C:\\Po-Yi\\Grad\\zPast\\10Projects\\CellSizeReg\\zzSmegData1605\\Smeg1605.csv','rt') as file:
#with open('PATH TO SMEG1605.CSV', 'rt') as file:
    reader = csv.reader(file)
    content = [row for row in reader]

n = 391
cn = [int(content[i][1]) for i in range(1, n)]
cind = [int(content[i][2]) for i in range(1, n)]
mom = [int(content[i][3]) for i in range(1, n)]
osa = [int(content[i][4]) for i in range(1, n)]
osb = [int(content[i][5]) for i in range(1, n)]
aa = [int(content[i][6]) for i in range(1, n)]
vb = [float(content[i][7]) for i in range(1, n)]
vi = [float(content[i][8]) for i in range(1, n)]
vd = [float(content[i][9]) for i in range(1, n)]
td = [float(content[i][11]) for i in range(1, n)]
b = [float(content[i][12]) for i in range(1, n)]
c = [float(content[i][13]) for i in range(1, n)]
d = [float(content[i][14]) for i in range(1, n)]
e = [float(content[i][15]) for i in range(1, n)]

vbs2 = [vb[i] for i in range(size(vb)) if aa[i] < 2]
vbs0 = [vb[i] for i in range(size(vb)) if aa[i] == 0]
vbs1 = [vb[i] for i in range(size(vb)) if aa[i] == 1]
tds2 = [td[i] for i in range(size(vb)) if aa[i] < 2]
tds0 = [td[i] for i in range(size(vb)) if aa[i] == 0]
tds1 = [td[i] for i in range(size(vb)) if aa[i] == 1]
vds2 = [vd[i] for i in range(size(vb)) if aa[i] < 2]
vds0 = [vd[i] for i in range(size(vb)) if aa[i] == 0]
vds1 = [vd[i] for i in range(size(vb)) if aa[i] == 1]
vis2 = [vi[i] for i in range(size(vb)) if aa[i] < 2]
vis0 = [vi[i] for i in range(size(vb)) if aa[i] == 0]
vis1 = [vi[i] for i in range(size(vb)) if aa[i] == 1]

c1 = [c[i] for i in range(size(vb)) if aa[i] == 1]
c0 = [c[i] for i in range(size(vb)) if aa[i] == 0]
c2 = [c[i] for i in range(size(vb)) if aa[i] < 2]
d1 = [d[i] for i in range(size(vb)) if aa[i] == 1]
d0 = [d[i] for i in range(size(vb)) if aa[i] == 0]
d2 = [d[i] for i in range(size(vb)) if aa[i] < 2]
e1 = [e[i] for i in range(size(vb)) if aa[i] == 1]
e0 = [e[i] for i in range(size(vb)) if aa[i] == 0]
e2 = [e[i] for i in range(size(vb)) if aa[i] < 2]


# In[6]:

#-----Generate plot comparing data with simulations
#-----Set parameters
r = 0.56
cvr = 0.03
bid0 = 0.85
bid1 = 1.09
bii0 = 0.91
bii1 = 1.15
cvt2 = 1e-12
cd = 0.978
cvcd = 0.2

#-----Read in data
x = array(vis2)
y = array(vds2)
y = y/mean(vbs2)
x = x/mean(vbs2)
y = y-x

#-----Make figure
from matplotlib.colors import ListedColormap
pairedCmap = ListedColormap(sns.color_palette("Paired"))
fig = figure(figsize=(6,6));
ax = fig.gca();

#-----Binning
bins = array(linspace(0.5,1.5,num=17)) #37C
binind = digitize(x, bins)
xbinned = array([x[binind == i].mean() for i in range(1, len(bins))])
ybinned = array([y[binind == i].mean() for i in range(1, len(bins))])
ybinnederr = array([y[binind == i].std()/sqrt(len(y[binind==i])) for i in range(1, len(bins))])

#-----Plot data and binned
s = ax.scatter(x, y, s=50, color=pairedCmap(0), marker='o', edgecolor='');
errorbar(xbinned, ybinned, xerr=0, yerr=ybinnederr, fmt='s', color=pairedCmap(1), markersize=10)

#-----Plot parallel adder
vbsS1, _, _, _, _, _, _, _, visrS1, vdsrS1, _, _, _, _ = parallelAdder(r, cvr, 0.10, 0.10, 2*0.09, cvt2, bid0, bid1, bii0, bii1)
x = array(visrS1)
y = array(vdsrS1)
y = y/mean(vbsS1)*(mean(vds2)/mean(vbs2)/2) #To correct for the discrete time measurements (see Methods)
x = x/mean(vbsS1)
y = y-x
zz = polyfit(x,y,1)
xx = linspace(0,2)
yy = polyval(zz,xx)
plot(xx, yy, '-', linewidth=4, color=pairedCmap(3), label='Parallel adder')

#-----Plot CH model
vbsS2, _, _, _, _, _, _, _, visrS2, vdsrS2, _, _, _, _ = chModel(r, cvr, 0.08, 0.05, 2*0.03, cvt2, cd, cvcd, bii0, bii1)
x = array(visrS2)
y = array(vdsrS2)
y = y/mean(vbsS2)*(mean(vds2)/mean(vbs2)/2)
x = x/mean(vbsS2)
y = y-x
zz = polyfit(x,y,1)
yy = polyval(zz,xx)
plot(xx, yy, '-', linewidth=4, color=pairedCmap(7), label='CH model')

#-----Label plots
legend(fontsize=14, loc=3, frameon=False);
xlabel(r'Size at initiation, $v_i$', fontsize=22)
ylabel(r'Increment to division, $v_d - v_i$', fontsize=22)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.get_xaxis().tick_bottom()
ax.get_yaxis().tick_left()
xlim([0.5,1.5])
ylim([0, 1.6])
xticks([0.5,0.75,1,1.25,1.5])
yticks([0, 0.4, 0.8, 1.2, 1.6])

#savefig('D:\Po-Yi\\Svivdi_37C.eps', bbox_inches='tight')

